@@ -24,28 +24,21 @@
 #include "config.h"
 
 #include <memory>
-#include <stdbool.h>
 
 #include <QObject>
-#include <QWidget>
 #include <QAbstractItemModel>
 #include <QAbstractItemView>
 #include <QString>
 #include <QPixmap>
-#include <QPainter>
 #include <QSet>
-#include <QStyleOption>
-#include <QStyledItemDelegate>
-#include <QStyleOptionViewItem>
-#include <QAction>
-#include <QMenu>
-#include <QtEvents>
 
 #include "core/song.h"
 #include "widgets/autoexpandingtreeview.h"
 
+class QWidget;
+class QMenu;
+class QAction;
 class QContextMenuEvent;
-class QHelpEvent;
 class QMouseEvent;
 class QPaintEvent;
 
@@ -60,7 +53,7 @@ class CollectionView : public AutoExpandingTreeView {
   Q_OBJECT
 
  public:
-  CollectionView(QWidget *parent = nullptr);
+  explicit CollectionView(QWidget *parent = nullptr);
   ~CollectionView();
 
   // Returns Songs currently selected in the collection view.
