@@ -26,10 +26,8 @@
 #include "config.h"
 
 #include <memory>
-#include <stdbool.h>
 
 #include <QObject>
-#include <QThread>
 #include <QList>
 #include <QString>
 
@@ -37,11 +35,13 @@
 
 using std::unique_ptr;
 
+class QThread;
+
 class TaskManager;
 class ApplicationImpl;
 class TagReaderClient;
 class Database;
-class EngineDevice;
+class DeviceFinders;
 class Player;
 class Appearance;
 class SCollection;
@@ -68,8 +68,6 @@ class Application : public QObject {
   Q_OBJECT
 
  public:
-  static bool kIsPortable;
-
   explicit Application(QObject *parent = nullptr);
   ~Application();
 
@@ -78,7 +76,7 @@ class Application : public QObject {
   Appearance *appearance() const;
   TaskManager *task_manager() const;
   Player *player() const;
-  EngineDevice *enginedevice() const;
+  DeviceFinders *device_finders() const;
 #ifndef Q_OS_WIN
   DeviceManager *device_manager() const;
 #endif
@@ -99,12 +97,6 @@ class Application : public QObject {
   AudioScrobbler *scrobbler() const;
 
   InternetServices *internet_services() const;
-#ifdef HAVE_TIDAL
-  InternetSearch *tidal_search() const;
-#endif
-#ifdef HAVE_QOBUZ
-  InternetSearch *qobuz_search() const;
-#endif
 
 #ifdef HAVE_MOODBAR
   MoodbarController *moodbar_controller() const;
@@ -116,7 +108,7 @@ class Application : public QObject {
   QThread *MoveToNewThread(QObject *object);
   void MoveToThread(QObject *object, QThread *thread);
 
-private slots:
+ private slots:
   void ExitReceived();
 
  public slots:
@@ -124,11 +116,12 @@ private slots:
   void ReloadSettings();
   void OpenSettingsDialogAtPage(SettingsDialog::Page page);
 
-signals:
+ signals:
   void ErrorAdded(const QString &message);
   void SettingsChanged();
   void SettingsDialogRequested(SettingsDialog::Page page);
   void ExitFinished();
+  void ClearPixmapDiskCache();
 
  private:
   std::unique_ptr<ApplicationImpl> p_;
