@@ -23,8 +23,6 @@
 
 #include "config.h"
 
-#include <stdbool.h>
-
 #include <QtGlobal>
 #include <QObject>
 #include <QList>
@@ -35,7 +33,6 @@
 #include <QString>
 #include <QStringList>
 #include <QImage>
-#include <QVector>
 #include <QtDBus>
 #include <QDBusArgument>
 #include <qdbusextratypes.h>
@@ -81,7 +78,7 @@ class Mpris2 : public QObject {
   Q_OBJECT
 
  public:
-  Mpris2(Application *app, QObject *parent = nullptr);
+  explicit Mpris2(Application *app, QObject *parent = nullptr);
 
   // org.mpris.MediaPlayer2 MPRIS 2.0 Root interface
   Q_PROPERTY(bool CanQuit READ CanQuit)
