@@ -27,12 +27,10 @@
 #include <taglib/id3v1genres.h>
 
 #include <QtGlobal>
-#include <QCoreApplication>
 #include <QObject>
 #include <QFile>
 #include <QFileInfo>
 #include <QSharedData>
-#include <QtAlgorithms>
 #include <QHash>
 #include <QByteArray>
 #include <QVariant>
@@ -152,12 +150,13 @@ const QString Song::kEmbeddedCover = "(embedded)";
 const QRegExp Song::kAlbumRemoveDisc(" ?-? ((\\(|\\[)?)(Disc|CD) ?([0-9]{1,2})((\\)|\\])?)$");
 const QRegExp Song::kAlbumRemoveMisc(" ?-? ((\\(|\\[)?)(Remastered|([0-9]{1,4}) *Remaster) ?((\\)|\\])?)$");
 const QRegExp Song::kTitleRemoveMisc(" ?-? ((\\(|\\[)?)(Remastered|Live|Remastered Version|([0-9]{1,4}) *Remaster) ?((\\)|\\])?)$");
+const QString Song::kVariousArtists("various artists");
 
 const QStringList Song::kArticles = QStringList() << "the " << "a " << "an ";
 
 struct Song::Private : public QSharedData {
 
-  Private(Source source = Source_Unknown);
+  explicit Private(Source source = Source_Unknown);
 
   bool valid_;
   int id_;
@@ -296,14 +295,15 @@ const QString &Song::albumartist() const { return d->albumartist_; }
 const QString &Song::albumartist_sortable() const { return d->albumartist_sortable_; }
 const QString &Song::effective_albumartist() const { return d->albumartist_.isEmpty() ? d->artist_ : d->albumartist_; }
 const QString &Song::effective_albumartist_sortable() const { return d->albumartist_.isEmpty() ? d->artist_sortable_ : d->albumartist_sortable_; }
-const QString &Song::playlist_albumartist() const { return is_compilation() ? d->albumartist_sortable_ : effective_albumartist_sortable(); }
+const QString &Song::playlist_albumartist() const { return is_compilation() ? d->albumartist_ : effective_albumartist(); }
+const QString &Song::playlist_albumartist_sortable() const { return is_compilation() ? d->albumartist_sortable_ : effective_albumartist_sortable(); }
 int Song::track() const { return d->track_; }
 int Song::disc() const { return d->disc_; }
 int Song::year() const { return d->year_; }
 int Song::originalyear() const { return d->originalyear_; }
 int Song::effective_originalyear() const { return d->originalyear_ < 0 ? d->year_ : d->originalyear_; }
 const QString &Song::genre() const { return d->genre_; }
-bool Song::is_compilation() const { return (d->compilation_ || d->compilation_detected_ || d->compilation_on_) && ! d->compilation_off_; }
+bool Song::compilation() const { return d->compilation_; }
 const QString &Song::composer() const { return d->composer_; }
 const QString &Song::performer() const { return d->performer_; }
 const QString &Song::grouping() const { return d->grouping_; }
@@ -331,6 +331,10 @@ int Song::playcount() const { return d->playcount_; }
 int Song::skipcount() const { return d->skipcount_; }
 int Song::lastplayed() const { return d->lastplayed_; }
 
+bool Song::compilation_detected() const { return d->compilation_detected_; }
+bool Song::compilation_off() const { return d->compilation_off_; }
+bool Song::compilation_on() const { return d->compilation_on_; }
+
 const QUrl &Song::art_automatic() const { return d->art_automatic_; }
 const QUrl &Song::art_manual() const { return d->art_manual_; }
 bool Song::has_manually_unset_cover() const { return d->art_manual_.path() == kManuallyUnsetCover; }
@@ -346,9 +350,10 @@ const QString &Song::cue_path() const { return d->cue_path_; }
 bool Song::has_cue() const { return !d->cue_path_.isEmpty(); }
 
 bool Song::is_collection_song() const { return d->source_ == Source_Collection; }
-bool Song::is_metadata_good() const { return !d->title_.isEmpty() && !d->album_.isEmpty() && !d->artist_.isEmpty() && !d->url_.isEmpty() && d->end_ > 0; }
+bool Song::is_metadata_good() const { return !d->title_.isEmpty() && !d->artist_.isEmpty() && !d->url_.isEmpty() && d->end_ > 0; }
 bool Song::is_stream() const { return d->source_ == Source_Stream || d->source_ == Source_Tidal || d->source_ == Source_Subsonic || d->source_ == Source_Qobuz; }
 bool Song::is_cdda() const { return d->source_ == Source_CDDA; }
+bool Song::is_compilation() const { return (d->compilation_ || d->compilation_detected_ || d->compilation_on_) && !d->compilation_off_; }
 
 bool Song::art_automatic_is_valid() const {
   return (
@@ -420,15 +425,7 @@ void Song::set_bitdepth(int v) { d->bitdepth_ = v; }
 
 void Song::set_source(Source v) { d->source_ = v; }
 void Song::set_directory_id(int v) { d->directory_id_ = v; }
-void Song::set_url(const QUrl &v) {
-  if (Application::kIsPortable) {
-    QUrl base = QUrl::fromLocalFile(QCoreApplication::applicationDirPath() + "/");
-    d->url_ = base.resolved(v);
-  }
-  else {
-    d->url_ = v;
-  }
-}
+void Song::set_url(const QUrl &v) { d->url_ = v; }
 void Song::set_basefilename(const QString &v) { d->basefilename_ = v; }
 void Song::set_filetype(FileType v) { d->filetype_ = v; }
 void Song::set_filesize(int v) { d->filesize_ = v; }
@@ -470,17 +467,17 @@ Song::Source Song::SourceFromURL(const QUrl &url) {
 QString Song::TextForSource(Source source) {
 
   switch (source) {
-    case Song::Source_LocalFile:   return QObject::tr("File");
-    case Song::Source_Collection:  return QObject::tr("Collection");
-    case Song::Source_CDDA:        return QObject::tr("CD");
-    case Song::Source_Device:      return QObject::tr("Device");
-    case Song::Source_Stream:      return QObject::tr("Stream");
-    case Song::Source_Tidal:       return QObject::tr("Tidal");
-    case Song::Source_Subsonic:    return QObject::tr("subsonic");
-    case Song::Source_Qobuz:       return QObject::tr("qobuz");
-    case Song::Source_Unknown:     return QObject::tr("Unknown");
+    case Song::Source_LocalFile:   return "file";
+    case Song::Source_Collection:  return "collection";
+    case Song::Source_CDDA:        return "cd";
+    case Song::Source_Device:      return "device";
+    case Song::Source_Stream:      return "stream";
+    case Song::Source_Tidal:       return "tidal";
+    case Song::Source_Subsonic:    return "subsonic";
+    case Song::Source_Qobuz:       return "qobuz";
+    case Song::Source_Unknown:     return "unknown";
   }
-  return QObject::tr("Unknown");
+  return "unknown";
 
 }
 
@@ -488,8 +485,8 @@ QIcon Song::IconForSource(Source source) {
 
   switch (source) {
     case Song::Source_LocalFile:   return IconLoader::Load("folder-sound");
-    case Song::Source_Collection:  return IconLoader::Load("vinyl");
-    case Song::Source_CDDA:        return IconLoader::Load("cd");
+    case Song::Source_Collection:  return IconLoader::Load("library-music");
+    case Song::Source_CDDA:        return IconLoader::Load("media-optical");
     case Song::Source_Device:      return IconLoader::Load("device");
     case Song::Source_Stream:      return IconLoader::Load("applications-internet");
     case Song::Source_Tidal:       return IconLoader::Load("tidal");
@@ -943,7 +940,7 @@ void Song::InitFromQuery(const SqlRow &q, bool reliable_metadata, int col) {
         set_art_automatic(QUrl::fromEncoded(art_automatic.toUtf8()));
       }
       else {
-        set_art_automatic(QUrl::fromLocalFile(art_automatic.toUtf8()));
+        set_art_automatic(QUrl::fromLocalFile(art_automatic));
       }
     }
     else if (Song::kColumns.value(i) == "art_manual") {
@@ -952,7 +949,7 @@ void Song::InitFromQuery(const SqlRow &q, bool reliable_metadata, int col) {
         set_art_manual(QUrl::fromEncoded(art_manual.toUtf8()));
       }
       else {
-        set_art_manual(QUrl::fromLocalFile(art_manual.toUtf8()));
+        set_art_manual(QUrl::fromLocalFile(art_manual));
       }
     }
 
@@ -1281,18 +1278,7 @@ void Song::BindToQuery(QSqlQuery *query) const {
 
   query->bindValue(":source", d->source_);
   query->bindValue(":directory_id", notnullintval(d->directory_id_));
-
-  QString url;
-  if (d->url_.isValid()) {
-    if (Application::kIsPortable && Utilities::UrlOnSameDriveAsStrawberry(d->url_)) {
-      url = Utilities::GetRelativePathToStrawberryBin(d->url_).toEncoded();
-    }
-    else {
-      url = d->url_.toEncoded();
-    }
-  }
-  query->bindValue(":url", url);
-
+  query->bindValue(":url", d->url_.toString(QUrl::FullyEncoded));
   query->bindValue(":filetype", d->filetype_);
   query->bindValue(":filesize", notnullintval(d->filesize_));
   query->bindValue(":mtime", notnullintval(d->mtime_));
@@ -1308,8 +1294,8 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":compilation_off", d->compilation_off_ ? 1 : 0);
   query->bindValue(":compilation_effective", is_compilation() ? 1 : 0);
 
-  query->bindValue(":art_automatic", d->art_automatic_);
-  query->bindValue(":art_manual", d->art_manual_);
+  query->bindValue(":art_automatic", d->art_automatic_.toString(QUrl::FullyEncoded));
+  query->bindValue(":art_manual", d->art_manual_.toString(QUrl::FullyEncoded));
 
   query->bindValue(":effective_albumartist", this->effective_albumartist());
   query->bindValue(":effective_originalyear", intval(this->effective_originalyear()));
