@@ -24,26 +24,26 @@
 
 #include "config.h"
 
-#include <stdbool.h>
 #include <memory>
 #include <libmtp.h>
 
 #include <QtGlobal>
+#include <QObject>
 #include <QList>
 #include <QUrl>
 
 #include "core/song.h"
 
 class MtpConnection : public QObject, public std::enable_shared_from_this<MtpConnection> {
-public:
-  MtpConnection(const QUrl &url);
+ public:
+  explicit MtpConnection(const QUrl &url);
   ~MtpConnection();
 
   bool is_valid() const { return device_; }
   LIBMTP_mtpdevice_t *device() const { return device_; }
   bool GetSupportedFiletypes(QList<Song::FileType> *ret);
 
-private:
+ private:
   Q_DISABLE_COPY(MtpConnection)
 
   LIBMTP_mtpdevice_t *device_;
