@@ -43,14 +43,12 @@
 #include <QStringList>
 #include <QUrl>
 #include <QTimeLine>
-#include <QTimerEvent>
 #include <QMetaObject>
 #include <QFlags>
-#include <QSettings>
+#include <QTimerEvent>
 #include <QtDebug>
 
 #include "core/closure.h"
-#include "core/utilities.h"
 #include "core/logging.h"
 #include "core/taskmanager.h"
 #include "core/timeconstants.h"
@@ -60,8 +58,6 @@
 #include "gstenginepipeline.h"
 #include "gstbufferconsumer.h"
 
-#include "settings/backendsettingspage.h"
-
 using std::shared_ptr;
 using std::vector;
 
@@ -83,7 +79,10 @@ GstEngine::GstEngine(TaskManager *task_manager)
       task_manager_(task_manager),
       buffering_task_id_(-1),
       latest_buffer_(nullptr),
+      stereo_balancer_enabled_(false),
       stereo_balance_(0.0f),
+      equalizer_enabled_(false),
+      equalizer_preamp_(0),
       seek_timer_(new QTimer(this)),
       timer_id_(-1),
       next_element_id_(0),
@@ -102,8 +101,15 @@ GstEngine::GstEngine(TaskManager *task_manager)
 }
 
 GstEngine::~GstEngine() {
+
   EnsureInitialised();
   current_pipeline_.reset();
+
+  if (latest_buffer_) {
+    gst_buffer_unref(latest_buffer_);
+    latest_buffer_ = nullptr;
+  }
+
 }
 
 bool GstEngine::Init() {
@@ -411,20 +417,34 @@ GstElement *GstEngine::CreateElement(const QString &factoryName, GstElement *bin
   return element;
 }
 
-void GstEngine::ConsumeBuffer(GstBuffer *buffer, const int pipeline_id) {
+void GstEngine::ConsumeBuffer(GstBuffer *buffer, const int pipeline_id, const QString &format) {
 
   // Schedule this to run in the GUI thread.  The buffer gets added to the queue and unreffed by UpdateScope.
-  if (!QMetaObject::invokeMethod(this, "AddBufferToScope", Q_ARG(GstBuffer*, buffer), Q_ARG(int, pipeline_id))) {
+  if (!QMetaObject::invokeMethod(this, "AddBufferToScope", Q_ARG(GstBuffer*, buffer), Q_ARG(int, pipeline_id), Q_ARG(QString, format))) {
     qLog(Warning) << "Failed to invoke AddBufferToScope on GstEngine";
   }
 
 }
 
+void GstEngine::SetStereoBalancerEnabled(const bool enabled) {
+
+  stereo_balancer_enabled_ = enabled;
+  if (current_pipeline_) current_pipeline_->set_stereo_balancer_enabled(enabled);
+
+}
+
+void GstEngine::SetStereoBalance(const float value) {
+
+  stereo_balance_ = value;
+  if (current_pipeline_) current_pipeline_->SetStereoBalance(value);
+
+}
+
 void GstEngine::SetEqualizerEnabled(const bool enabled) {
 
   equalizer_enabled_ = enabled;
+  if (current_pipeline_) current_pipeline_->set_equalizer_enabled(enabled);
 
-  if (current_pipeline_) current_pipeline_->SetEqualizerEnabled(enabled);
 }
 
 void GstEngine::SetEqualizerParameters(const int preamp, const QList<int> &band_gains) {
@@ -432,27 +452,22 @@ void GstEngine::SetEqualizerParameters(const int preamp, const QList<int> &band_
   equalizer_preamp_ = preamp;
   equalizer_gains_ = band_gains;
 
-  if (current_pipeline_)
-    current_pipeline_->SetEqualizerParams(preamp, band_gains);
-
-}
-
-void GstEngine::SetStereoBalance(const float value) {
-
-  stereo_balance_ = value;
-
-  if (current_pipeline_) current_pipeline_->SetStereoBalance(value);
+  if (current_pipeline_) current_pipeline_->SetEqualizerParams(preamp, band_gains);
 
 }
 
 void GstEngine::AddBufferConsumer(GstBufferConsumer *consumer) {
+
   buffer_consumers_ << consumer;
   if (current_pipeline_) current_pipeline_->AddBufferConsumer(consumer);
+
 }
 
 void GstEngine::RemoveBufferConsumer(GstBufferConsumer *consumer) {
+
   buffer_consumers_.removeAll(consumer);
   if (current_pipeline_) current_pipeline_->RemoveBufferConsumer(consumer);
+
 }
 
 void GstEngine::timerEvent(QTimerEvent *e) {
@@ -492,7 +507,7 @@ void GstEngine::EndOfStreamReached(const int pipeline_id, const bool has_next_tr
 
 }
 
-void GstEngine::HandlePipelineError(int pipeline_id, const QString &message, int domain, int error_code) {
+void GstEngine::HandlePipelineError(const int pipeline_id, const QString &message, const int domain, const int error_code) {
 
   if (!current_pipeline_.get() || current_pipeline_->id() != pipeline_id) return;
 
@@ -502,7 +517,7 @@ void GstEngine::HandlePipelineError(int pipeline_id, const QString &message, int
   BufferingFinished();
   emit StateChanged(Engine::Error);
 
-  if (domain == GST_RESOURCE_ERROR && (error_code == GST_RESOURCE_ERROR_NOT_FOUND || error_code == GST_RESOURCE_ERROR_NOT_AUTHORIZED)) {
+  if (domain == (int)GST_RESOURCE_ERROR && (error_code == (int)GST_RESOURCE_ERROR_NOT_FOUND || error_code == (int)GST_RESOURCE_ERROR_NOT_AUTHORIZED)) {
      emit InvalidSongRequested(stream_url_);
    }
   else {
@@ -520,7 +535,7 @@ void GstEngine::NewMetaData(const int pipeline_id, const Engine::SimpleMetaBundl
 
 }
 
-void GstEngine::AddBufferToScope(GstBuffer *buf, const int pipeline_id) {
+void GstEngine::AddBufferToScope(GstBuffer *buf, const int pipeline_id, const QString &format) {
 
   if (!current_pipeline_ || current_pipeline_->id() != pipeline_id) {
     gst_buffer_unref(buf);
@@ -531,6 +546,7 @@ void GstEngine::AddBufferToScope(GstBuffer *buf, const int pipeline_id) {
     gst_buffer_unref(latest_buffer_);
   }
 
+  buffer_format_ = format;
   latest_buffer_ = buf;
   have_new_buffer_ = true;
 
@@ -739,11 +755,12 @@ shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline() {
 
   shared_ptr<GstEnginePipeline> ret(new GstEnginePipeline(this));
   ret->set_output_device(output_, device_);
-  ret->set_volume_control(volume_control_);
+  ret->set_volume_enabled(volume_control_);
+  ret->set_stereo_balancer_enabled(stereo_balancer_enabled_);
+  ret->set_equalizer_enabled(equalizer_enabled_);
   ret->set_replaygain(rg_enabled_, rg_mode_, rg_preamp_, rg_compression_);
   ret->set_buffer_duration_nanosec(buffer_duration_nanosec_);
   ret->set_buffer_min_fill(buffer_min_fill_);
-  ret->SetEqualizerEnabled(equalizer_enabled_);
 
   ret->AddBufferConsumer(this);
   for (GstBufferConsumer *consumer : buffer_consumers_) {
@@ -806,13 +823,22 @@ void GstEngine::UpdateScope(const int chunk_length) {
   }
 
   scope_chunk_++;
-  memcpy(dest, source, bytes);
+
+  if (buffer_format_.startsWith("S16") ||
+      buffer_format_.startsWith("U16") ||
+      buffer_format_.startsWith("S32")) {
+    memcpy(dest, source, bytes);
+  }
+  else {
+    memset(dest, 0, bytes);
+  }
 
   gst_buffer_unmap(latest_buffer_, &map);
 
   if (scope_chunk_ == scope_chunks_) {
     gst_buffer_unref(latest_buffer_);
     latest_buffer_ = nullptr;
+    buffer_format_.clear();
   }
 
 }
