@@ -23,12 +23,7 @@
 
 #include "config.h"
 
-#include <stdbool.h>
-
-#include <QtGlobal>
 #include <QObject>
-#include <QPair>
-#include <QHash>
 #include <QString>
 
 class GlobalShortcuts;
