@@ -20,8 +20,7 @@
 
 #include "config.h"
 
-#include <stdbool.h>
-
+#include <QAbstractItemModel>
 #include <QWidget>
 #include <QFlags>
 #include <QVariant>
@@ -30,8 +29,8 @@
 #include <QMenu>
 #include <QAction>
 #include <QActionGroup>
+#include <QEvent>
 #include <QContextMenuEvent>
-#include <QtEvents>
 
 #include "playlistheader.h"
 #include "playlistview.h"
