@@ -19,12 +19,14 @@
 
 #include "config.h"
 
-#include <stdbool.h>
+#include <algorithm>
 
 #include <QtGlobal>
 #include <QApplication>
 #include <QDesktopServices>
+#include <QLocale>
 #include <QClipboard>
+#include <QPair>
 #include <QVariant>
 #include <QByteArray>
 #include <QString>
@@ -32,16 +34,16 @@
 #include <QUrlQuery>
 #include <QDateTime>
 #include <QCryptographicHash>
-#include <QMenu>
 #include <QMessageBox>
 #include <QSettings>
 #include <QNetworkRequest>
 #include <QNetworkReply>
-#include <QJsonParseError>
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonArray>
 #include <QJsonValue>
+#include <QFlags>
+#include <QtDebug>
 
 #include "core/application.h"
 #include "core/closure.h"
@@ -50,11 +52,10 @@
 #include "core/timeconstants.h"
 #include "core/logging.h"
 #include "internet/localredirectserver.h"
-#include "settings/settingsdialog.h"
 #include "settings/scrobblersettingspage.h"
 
 #include "audioscrobbler.h"
-#include "scrobblerservices.h"
+#include "scrobblerservice.h"
 #include "scrobblingapi20.h"
 #include "scrobblercache.h"
 #include "scrobblercacheitem.h"
@@ -72,20 +73,27 @@ ScrobblingAPI20::ScrobblingAPI20(const QString &name, const QString &settings_gr
   api_url_(api_url),
   batch_(batch),
   app_(app),
+  server_(nullptr),
   enabled_(false),
   subscriber_(false),
-  submitted_(false) {}
+  submitted_(false),
+  timestamp_(0) {}
 
 ScrobblingAPI20::~ScrobblingAPI20() {}
 
 void ScrobblingAPI20::ReloadSettings() {
 
   QSettings s;
+
   s.beginGroup(settings_group_);
   enabled_ = s.value("enabled", false).toBool();
   https_ = s.value("https", false).toBool();
   s.endGroup();
 
+  s.beginGroup(ScrobblerSettingsPage::kSettingsGroup);
+  prefer_albumartist_ = s.value("albumartist", false).toBool();
+  s.endGroup();
+
 }
 
 void ScrobblingAPI20::LoadSession() {
@@ -116,16 +124,19 @@ void ScrobblingAPI20::Logout() {
 
 void ScrobblingAPI20::Authenticate(const bool https) {
 
-  LocalRedirectServer *server = new LocalRedirectServer(https, this);
-  if (!server->Listen()) {
-    AuthError(server->error());
-    delete server;
-    return;
+  if (!server_) {
+    server_ = new LocalRedirectServer(https, this);
+    if (!server_->Listen()) {
+      AuthError(server_->error());
+      delete server_;
+      server_ = nullptr;
+      return;
+    }
+    connect(server_, SIGNAL(Finished()), this, SLOT(RedirectArrived()));
   }
-  NewClosure(server, SIGNAL(Finished()), this, &ScrobblingAPI20::RedirectArrived, server);
 
   QUrlQuery redirect_url_query;
-  const QString port = QString::number(server->url().port());
+  const QString port = QString::number(server_->url().port());
   redirect_url_query.addQueryItem("port", port);  
   if (https) redirect_url_query.addQueryItem("https", QString("1"));
   QUrl redirect_url(kRedirectUrl);
@@ -137,7 +148,7 @@ void ScrobblingAPI20::Authenticate(const bool https) {
   QUrl url(auth_url_);
   url.setQuery(url_query);
 
-  QMessageBox messagebox(QMessageBox::Information, tr("%1 Scrobbler Authentication").arg(name_), tr("Open URL in web browser?<br /><a href=\"%1\">%1</a><br />Press \"Save\" to copy the URL to clipboard and manually open it in a web browser.").arg(url.toString()), QMessageBox::Open|QMessageBox::Save|QMessageBox::Cancel);
+  QMessageBox messagebox(QMessageBox::Information, tr("%1 Scrobbler Authentication").arg(name_), tr("Open URL in web browser?") + QString("<br /><a href=\"%1\">%1</a><br />").arg(url.toString()) + tr("Press \"Save\" to copy the URL to clipboard and manually open it in a web browser."), QMessageBox::Open|QMessageBox::Save|QMessageBox::Cancel);
   messagebox.setTextFormat(Qt::RichText);
   int result = messagebox.exec();
   switch (result) {
@@ -146,7 +157,7 @@ void ScrobblingAPI20::Authenticate(const bool https) {
       if (openurl_result) {
         break;
       }
-      QMessageBox messagebox_error(QMessageBox::Warning, tr("%1 Scrobbler Authentication").arg(name_), tr("Could not open URL. Please open this URL in your browser:<br /><a href=\"%1\">%1</a>").arg(url.toString()), QMessageBox::Ok);
+      QMessageBox messagebox_error(QMessageBox::Warning, tr("%1 Scrobbler Authentication").arg(name_), tr("Could not open URL. Please open this URL in your browser") + QString(":<br /><a href=\"%1\">%1</a>").arg(url.toString()), QMessageBox::Ok);
       messagebox_error.setTextFormat(Qt::RichText);
       messagebox_error.exec();
     }
@@ -155,8 +166,11 @@ void ScrobblingAPI20::Authenticate(const bool https) {
     QApplication::clipboard()->setText(url.toString());
     break;
   case QMessageBox::Cancel:
-    server->close();
-    server->deleteLater();
+    if (server_) {
+      server_->close();
+      server_->deleteLater();
+      server_ = nullptr;
+    }
     emit AuthenticationComplete(false);
     break;
   default:
@@ -165,31 +179,37 @@ void ScrobblingAPI20::Authenticate(const bool https) {
 
 }
 
-void ScrobblingAPI20::RedirectArrived(LocalRedirectServer *server) {
+void ScrobblingAPI20::RedirectArrived() {
 
-  server->deleteLater();
+  if (!server_) return;
 
-  if (!server->error().isEmpty()) {
-    AuthError(server->error());
-    return;
+  if (server_->error().isEmpty()) {
+    QUrl url = server_->request_url();
+    if (url.isValid()) {
+      QUrlQuery url_query(url);
+      if (url_query.hasQueryItem("token")) {
+        QString token = url_query.queryItemValue("token").toUtf8();
+        RequestSession(token);
+      }
+      else {
+        AuthError(tr("Invalid reply from web browser. Missing token."));
+      }
+    }
+    else {
+      AuthError(tr("Received invalid reply from web browser. Try the HTTPS option, or use another browser like Chromium or Chrome."));
+    }
+  }
+  else {
+    AuthError(server_->error());
   }
 
-  QUrl url = server->request_url();
-  if (!url.isValid()) {
-    AuthError(tr("Received invalid reply from web browser. Try the HTTPS option, or use another browser like Chromium or Chrome."));
-    return;
-  }
-  QUrlQuery url_query(url);
-  QString token = url_query.queryItemValue("token").toUtf8();
-  if (token.isEmpty()) {
-    AuthError(tr("Invalid reply from web browser. Missing token."));
-    return;
-  }
-  RequestSession(token);
+  server_->close();
+  server_->deleteLater();
+  server_ = nullptr;
 
 }
 
-void ScrobblingAPI20::RequestSession(QString token) {
+void ScrobblingAPI20::RequestSession(const QString &token) {
 
   QUrl session_url(api_url_);
   QUrlQuery session_url_query;
@@ -208,9 +228,7 @@ void ScrobblingAPI20::RequestSession(QString token) {
   session_url.setQuery(session_url_query);
 
   QNetworkRequest req(session_url);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
   req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
-#endif
   QNetworkReply *reply = network()->get(req);
   NewClosure(reply, SIGNAL(finished()), this, SLOT(AuthenticateReplyFinished(QNetworkReply*)), reply);
 
@@ -338,9 +356,7 @@ QNetworkReply *ScrobblingAPI20::CreateRequest(const ParamList &request_params) {
 
   QUrl url(api_url_);
   QNetworkRequest req(url);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
   req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
-#endif
   req.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
   QByteArray query = url_query.toString(QUrl::FullyEncoded).toUtf8();
   QNetworkReply *reply = network()->post(req, query);
@@ -417,10 +433,14 @@ void ScrobblingAPI20::UpdateNowPlaying(const Song &song) {
 
   ParamList params = ParamList()
     << Param("method", "track.updateNowPlaying")
-    << Param("artist", song.artist())
-    << Param("track", song.title())
-    << Param("album", album);
-  if (!song.albumartist().isEmpty()) params << Param("albumArtist", song.albumartist());
+    << Param("artist", prefer_albumartist_ && song.effective_albumartist() != Song::kVariousArtists ? song.effective_albumartist() : song.artist())
+    << Param("track", song.title());
+
+  if (!album.isEmpty())
+    params << Param("album", album);
+
+  if (!prefer_albumartist_ && !song.albumartist().isEmpty() && song.albumartist().toLower() != Song::kVariousArtists)
+    params << Param("albumArtist", song.albumartist());
 
   QNetworkReply *reply = CreateRequest(params);
   NewClosure(reply, SIGNAL(finished()), this, SLOT(UpdateNowPlayingRequestFinished(QNetworkReply*)), reply);
@@ -512,16 +532,22 @@ void ScrobblingAPI20::Submit() {
   for (ScrobblerCacheItem *item : cache()->List()) {
     if (item->sent_) continue;
     item->sent_ = true;
-    if (!batch_) { SendSingleScrobble(item); continue; }
+    if (!batch_) {
+      SendSingleScrobble(item);
+      continue;
+    }
     i++;
     list << item->timestamp_;
-    params << Param(QString("%1[%2]").arg("artist").arg(i), item->artist_);
-    params << Param(QString("%1[%2]").arg("album").arg(i), item->album_);
+    params << Param(QString("%1[%2]").arg("artist").arg(i), prefer_albumartist_ ? item->effective_albumartist() : item->artist_);
     params << Param(QString("%1[%2]").arg("track").arg(i), item->song_);
     params << Param(QString("%1[%2]").arg("timestamp").arg(i), QString::number(item->timestamp_));
     params << Param(QString("%1[%2]").arg("duration").arg(i), QString::number(item->duration_ / kNsecPerSec));
-    if (!item->albumartist_.isEmpty()) params << Param(QString("%1[%2]").arg("albumArtist").arg(i), item->albumartist_);
-    if (item->track_ > 0) params << Param(QString("%1[%2]").arg(i).arg("trackNumber"), QString::number(item->track_));
+    if (!item->album_.isEmpty())
+      params << Param(QString("%1[%2]").arg("album").arg(i), item->album_);
+    if (!prefer_albumartist_ && !item->albumartist_.isEmpty() && item->albumartist_.toLower() != Song::kVariousArtists)
+      params << Param(QString("%1[%2]").arg("albumArtist").arg(i), item->albumartist_);
+    if (item->track_ > 0)
+      params << Param(QString("%1[%2]").arg(i).arg("trackNumber"), QString::number(item->track_));
     if (i >= kScrobblesPerRequest) break;
   }
 
@@ -692,14 +718,17 @@ void ScrobblingAPI20::SendSingleScrobble(ScrobblerCacheItem *item) {
 
   ParamList params = ParamList()
     << Param("method", "track.scrobble")
-    << Param("artist", item->artist_)
-    << Param("album", item->album_)
+    << Param("artist", prefer_albumartist_ ? item->effective_albumartist() : item->artist_)
     << Param("track", item->song_)
     << Param("timestamp", QString::number(item->timestamp_))
     << Param("duration", QString::number(item->duration_ / kNsecPerSec));
 
-  if (!item->albumartist_.isEmpty()) params << Param("albumArtist", item->albumartist_);
-  if (item->track_ > 0) params << Param("trackNumber", QString::number(item->track_));
+  if (!item->album_.isEmpty())
+    params << Param("album", item->album_);
+  if (!prefer_albumartist_ && !item->albumartist_.isEmpty() && item->albumartist_.toLower() != Song::kVariousArtists)
+    params << Param("albumArtist", item->albumartist_);
+  if (item->track_ > 0)
+    params << Param("trackNumber", QString::number(item->track_));
 
   QNetworkReply *reply = CreateRequest(params);
   NewClosure(reply, SIGNAL(finished()), this, SLOT(SingleScrobbleRequestFinished(QNetworkReply*, quint64)), reply, item->timestamp_);
@@ -840,10 +869,14 @@ void ScrobblingAPI20::Love() {
 
   ParamList params = ParamList()
     << Param("method", "track.love")
-    << Param("artist", song_playing_.artist())
-    << Param("track", song_playing_.title())
-    << Param("album", song_playing_.album());
-  if (!song_playing_.albumartist().isEmpty()) params << Param("albumArtist", song_playing_.albumartist());
+    << Param("artist", prefer_albumartist_ && song_playing_.effective_albumartist() != Song::kVariousArtists ? song_playing_.effective_albumartist() : song_playing_.artist())
+    << Param("track", song_playing_.title());
+
+  if (!song_playing_.album().isEmpty())
+    params << Param("album", song_playing_.album());
+
+  if (!prefer_albumartist_ && !song_playing_.albumartist().isEmpty() && song_playing_.albumartist().toLower() != Song::kVariousArtists)
+    params << Param("albumArtist", song_playing_.albumartist());
 
   QNetworkReply *reply = CreateRequest(params);
   NewClosure(reply, SIGNAL(finished()), this, SLOT(LoveRequestFinished(QNetworkReply*)), reply);
@@ -900,18 +933,18 @@ void ScrobblingAPI20::LoveRequestFinished(QNetworkReply *reply) {
 
 }
 
-void ScrobblingAPI20::AuthError(QString error) {
+void ScrobblingAPI20::AuthError(const QString &error) {
   emit AuthenticationComplete(false, error);
 }
 
-void ScrobblingAPI20::Error(QString error, QVariant debug) {
+void ScrobblingAPI20::Error(const QString &error, const QVariant &debug) {
 
   qLog(Error) << name_ << error;
   if (debug.isValid()) qLog(Debug) << debug;
 
 }
 
-QString ScrobblingAPI20::ErrorString(ScrobbleErrorCode error) const {
+QString ScrobblingAPI20::ErrorString(const ScrobbleErrorCode error) const {
 
   switch (error) {
     case ScrobbleErrorCode::InvalidService:
